package javax.swing.text.html;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.EventListenerList;
import java.util.BitSet;
import java.io.Serializable;
import javax.swing.ListSelectionModel;
import javax.swing.DefaultListModel;

class OptionListModel<E> extends DefaultListModel<E> implements ListSelectionModel, Serializable
{
    private static final int MIN = -1;
    private static final int MAX = Integer.MAX_VALUE;
    private int selectionMode;
    private int minIndex;
    private int maxIndex;
    private int anchorIndex;
    private int leadIndex;
    private int firstChangedIndex;
    private int lastChangedIndex;
    private boolean isAdjusting;
    private BitSet value;
    private BitSet initialValue;
    protected EventListenerList listenerList;
    protected boolean leadAnchorNotificationEnabled;
    
    OptionListModel() {
        this.selectionMode = 0;
        this.minIndex = Integer.MAX_VALUE;
        this.maxIndex = -1;
        this.anchorIndex = -1;
        this.leadIndex = -1;
        this.firstChangedIndex = Integer.MAX_VALUE;
        this.lastChangedIndex = -1;
        this.isAdjusting = false;
        this.value = new BitSet(32);
        this.initialValue = new BitSet(32);
        this.listenerList = new EventListenerList();
        this.leadAnchorNotificationEnabled = true;
    }
    
    @Override
    public int getMinSelectionIndex() {
        return this.isSelectionEmpty() ? -1 : this.minIndex;
    }
    
    @Override
    public int getMaxSelectionIndex() {
        return this.maxIndex;
    }
    
    @Override
    public boolean getValueIsAdjusting() {
        return this.isAdjusting;
    }
    
    @Override
    public int getSelectionMode() {
        return this.selectionMode;
    }
    
    @Override
    public void setSelectionMode(int selectionMode) {
        switch (selectionMode) {
            case 0:
            case 1:
            case 2: {
                this.selectionMode = selectionMode;
                return;
            }
            default: {
                throw new IllegalArgumentException("invalid selectionMode");
            }
        }
    }
    
    @Override
    public boolean isSelectedIndex(int bitIndex) {
        return bitIndex >= this.minIndex && bitIndex <= this.maxIndex && this.value.get(bitIndex);
    }
    
    @Override
    public boolean isSelectionEmpty() {
        return this.minIndex > this.maxIndex;
    }
    
    @Override
    public void addListSelectionListener(ListSelectionListener l) {
        this.listenerList.add(ListSelectionListener.class, l);
    }
    
    @Override
    public void removeListSelectionListener(ListSelectionListener l) {
        this.listenerList.remove(ListSelectionListener.class, l);
    }
    
    public ListSelectionListener[] getListSelectionListeners() {
        return (ListSelectionListener[])this.listenerList.getListeners(ListSelectionListener.class);
    }
    
    protected void fireValueChanged(boolean b) {
        this.fireValueChanged(this.getMinSelectionIndex(), this.getMaxSelectionIndex(), b);
    }
    
    protected void fireValueChanged(int n, int n2) {
        this.fireValueChanged(n, n2, this.getValueIsAdjusting());
    }
    
    protected void fireValueChanged(int firstIndex, int lastIndex, boolean isAdjusting) {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ListSelectionListener.class) {
                if (o == null) {
                    o = new ListSelectionEvent(this, firstIndex, lastIndex, isAdjusting);
                }
                ((ListSelectionListener)listenerList[i + 1]).valueChanged(o);
            }
        }
    }
    
    private void fireValueChanged() {
        if (this.lastChangedIndex == -1) {
            return;
        }
        int firstChangedIndex = this.firstChangedIndex;
        int lastChangedIndex = this.lastChangedIndex;
        this.firstChangedIndex = Integer.MAX_VALUE;
        this.lastChangedIndex = -1;
        this.fireValueChanged(firstChangedIndex, lastChangedIndex);
        return;
    }
    
    private void markAsDirty(int n) {
        this.firstChangedIndex = Math.min(this.firstChangedIndex, n);
        this.lastChangedIndex = Math.max(this.lastChangedIndex, n);
    }
    
    private void set(int b) {
        if (this.value.get(b)) {
            return;
        }
        this.value.set(b);
        ((Option)this.get(b)).setSelection(true);
        this.markAsDirty(b);
        this.minIndex = Math.min(this.minIndex, b);
        this.maxIndex = Math.max(this.maxIndex, b);
        return;
    }
    
    private void clear(int index) {
        if (!this.value.get(index)) {
            return;
        }
        this.value.clear(index);
        ((Option)this.get(index)).setSelection(false);
        this.markAsDirty(index);
        if (index == this.minIndex) {
            ++this.minIndex;
            while (this.minIndex <= this.maxIndex) {
                if (this.value.get(this.minIndex)) {
                    break;
                }
                ++this.minIndex;
            }
        }
        if (index == this.maxIndex) {
            --this.maxIndex;
            while (this.minIndex <= this.maxIndex) {
                if (this.value.get(this.maxIndex)) {
                    break;
                }
                --this.maxIndex;
            }
        }
        if (this.isSelectionEmpty()) {
            this.minIndex = Integer.MAX_VALUE;
            this.maxIndex = -1;
        }
        return;
    }
    
    public void setLeadAnchorNotificationEnabled(boolean leadAnchorNotificationEnabled) {
        this.leadAnchorNotificationEnabled = leadAnchorNotificationEnabled;
    }
    
    public boolean isLeadAnchorNotificationEnabled() {
        return this.leadAnchorNotificationEnabled;
    }
    
    private void updateLeadAnchorIndices(int anchorIndex, int leadIndex) {
        if (this.leadAnchorNotificationEnabled) {
            if (this.anchorIndex != anchorIndex) {
                if (this.anchorIndex != -1) {
                    this.markAsDirty(this.anchorIndex);
                }
                this.markAsDirty(anchorIndex);
            }
            if (this.leadIndex != leadIndex) {
                if (this.leadIndex != -1) {
                    this.markAsDirty(this.leadIndex);
                }
                this.markAsDirty(leadIndex);
            }
        }
        this.anchorIndex = anchorIndex;
        this.leadIndex = leadIndex;
    }
    
    private boolean contains(int n, int n2, int n3) {
        return n3 >= n && n3 <= n2;
    }
    
    private void changeSelection(int b, int b2, int a, int a2, boolean b3) {
        for (int i = Math.min(a, b); i <= Math.max(a2, b2); ++i) {
            int contains = this.contains(b, b2, i);
            int contains2 = this.contains(a, a2, i);
            if (contains2 != 0 && contains != 0) {
                if (b3) {
                    contains = 0;
                }
                else {
                    contains2 = 0;
                }
            }
            if (contains2 != 0) {
                this.set(i);
            }
            if (contains != 0) {
                this.clear(i);
            }
        }
        this.fireValueChanged();
    }
    
    private void changeSelection(int n, int n2, int n3, int n4) {
        this.changeSelection(n, n2, n3, n4, true);
    }
    
    @Override
    public void clearSelection() {
        this.removeSelectionInterval(this.minIndex, this.maxIndex);
    }
    
    @Override
    public void setSelectionInterval(int n, int n2) {
        if (n == -1 || n2 == -1) {
            return;
        }
        if (this.getSelectionMode() == 0) {
            n = n2;
        }
        this.updateLeadAnchorIndices(n, n2);
        this.changeSelection(this.minIndex, this.maxIndex, Math.min(n, n2), Math.max(n, n2));
        return;
    }
    
    @Override
    public void addSelectionInterval(int n, int n2) {
        if (n == -1 || n2 == -1) {
            return;
        }
        if (this.getSelectionMode() != 2) {
            this.setSelectionInterval(n, n2);
            return;
        }
        this.updateLeadAnchorIndices(n, n2);
        this.changeSelection(Integer.MAX_VALUE, -1, Math.min(n, n2), Math.max(n, n2));
        return;
    }
    
    @Override
    public void removeSelectionInterval(int n, int n2) {
        if (n == -1 || n2 == -1) {
            return;
        }
        this.updateLeadAnchorIndices(n, n2);
        this.changeSelection(Math.min(n, n2), Math.max(n, n2), Integer.MAX_VALUE, -1);
        return;
    }
    
    private void setState(int n, boolean b) {
        if (b) {
            this.set(n);
        }
        else {
            this.clear(n);
        }
    }
    
    @Override
    public void insertIndexInterval(int bitIndex, int n, boolean b) {
        int i = b ? bitIndex : (bitIndex + 1);
        int n2 = i + n - 1;
        for (int j = this.maxIndex; j >= i; --j) {
            this.setState(j + n, this.value.get(j));
        }
        int value = this.value.get(bitIndex);
        for (; i <= n2; ++i) {
            this.setState(i, (boolean)value);
        }
    }
    
    @Override
    public void removeIndexInterval(int n, int n2) {
        int i = Math.min(n, n2);
        int n3 = Math.max(n, n2) - i + 1;
        for (; i <= this.maxIndex; ++i) {
            this.setState(i, this.value.get(i + n3));
        }
    }
    
    @Override
    public void setValueIsAdjusting(boolean isAdjusting) {
        if (isAdjusting != this.isAdjusting) {
            this.fireValueChanged(this.isAdjusting = isAdjusting);
        }
    }
    
    @Override
    public String toString() {
        return this.getClass().getName() + " " + Integer.toString(this.hashCode()) + " " + ((this.getValueIsAdjusting() ? "~" : "=") + this.value.toString());
    }
    
    public Object clone() throws CloneNotSupportedException {
        OptionListModel<Object> optionListModel = (OptionListModel)super.clone();
        optionListModel.value = (BitSet)this.value.clone();
        optionListModel.listenerList = new EventListenerList();
        return optionListModel;
    }
    
    @Override
    public int getAnchorSelectionIndex() {
        return this.anchorIndex;
    }
    
    @Override
    public int getLeadSelectionIndex() {
        return this.leadIndex;
    }
    
    @Override
    public void setAnchorSelectionIndex(int anchorIndex) {
        this.anchorIndex = anchorIndex;
    }
    
    @Override
    public void setLeadSelectionIndex(int leadIndex) {
        int anchorIndex = this.anchorIndex;
        if (this.getSelectionMode() == 0) {
            anchorIndex = leadIndex;
        }
        int min = Math.min(this.anchorIndex, this.leadIndex);
        int max = Math.max(this.anchorIndex, this.leadIndex);
        int min2 = Math.min(anchorIndex, leadIndex);
        int max2 = Math.max(anchorIndex, leadIndex);
        if (this.value.get(this.anchorIndex)) {
            this.changeSelection(min, max, min2, max2);
        }
        else {
            this.changeSelection(min2, max2, min, max, false);
        }
        this.anchorIndex = anchorIndex;
        this.leadIndex = leadIndex;
    }
    
    public void setInitialSelection(int n) {
        if (this.initialValue.get(n)) {
            return;
        }
        if (this.selectionMode == 0) {
            this.initialValue.and(new BitSet());
        }
        this.initialValue.set(n);
        return;
    }
    
    public BitSet getInitialSelection() {
        return this.initialValue;
    }
}
